package com.mamezou.sample.business;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanCreationException;
import org.springframework.beans.factory.BeanDefinitionStoreException;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.AbstractBeanFactory;
import org.springframework.beans.factory.support.RootBeanDefinition;

/**
 * Tvr[t@NgNX
 * @author MBeans
 * @version 1.0
 */
public class SampleBeanFactory extends AbstractBeanFactory {

    private Map<String, BeanDefinition> beans;
    private Map<String, Object> singletons;

    public SampleBeanFactory() {
        beans = new HashMap<String, BeanDefinition>();
        for (BeanEntry beanEntry : getBeanEntries()) {
            beans.put(beanEntry.getBeanName(), beanEntry.getBeanDefinition());
        }
        singletons = new HashMap<String, Object>();
    }

    protected Set<BeanEntry> getBeanEntries() {
        Set<BeanEntry> beanEntries = new HashSet<BeanEntry>();
        beanEntries.add(createBeanEntry(
                "sampleManager",
                "com.mamezou.sample.business.SampleManagerImpl", true));
        return beanEntries;
    }

    private BeanEntry createBeanEntry(
            String beanName, String className, boolean singleton) {
        return new BeanEntry(
                beanName, createRootBeanDefinition(className, singleton));
    }

    private BeanDefinition createRootBeanDefinition(
            String className, boolean singleton) {
        Class beanClass = null;
        try {
            beanClass = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new BeanDefinitionStoreException("NOT HAPPEN", e);
        }
        return new RootBeanDefinition(beanClass, singleton);
    }

    protected static final class BeanEntry {
        private String beanName;
        private BeanDefinition beanDefinition;
        BeanEntry(String beanName, BeanDefinition beanDefinition) {
            this.beanName = beanName;
            this.beanDefinition = beanDefinition;
        }
        String getBeanName() {
            return beanName;
        }
        BeanDefinition getBeanDefinition() {
            return beanDefinition;
        }
        public boolean equals(Object other) {
            if (!(other instanceof BeanEntry)) {
                return false;
            }
            return this.beanName.equals(((BeanEntry) other).beanName);
        }
        public int hashCode() {
            int result = 17;
            result = result * 37 + beanName.hashCode();
            return result;
        }
    }

    protected boolean containsBeanDefinition(String beanName) {
        return beans.containsKey(beanName);
    }

    protected BeanDefinition getBeanDefinition(String beanName)
            throws BeansException {
        return beans.get(beanName);
    }

    protected Object createBean(
            String beanName, RootBeanDefinition mergedBeanDefinition,
            Object[] args) throws BeanCreationException {
        if (mergedBeanDefinition.isSingleton()) {
            synchronized (this) {
                if (!singletons.containsKey(beanName)) {
                    singletons.put(beanName, createBean(mergedBeanDefinition));
                }
                return singletons.get(beanName);
            }
        }
        return createBean(mergedBeanDefinition);
    }

    private Object createBean(RootBeanDefinition rootBeanDefinition) {
        try {
            return rootBeanDefinition.getBeanClass().newInstance();
        } catch (Exception e) {
            throw new BeanCreationException("NOT HAPPEN", e);
        }
    }
}
